/* src/admin_chan.c - commandes admin pour gerer les salons
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "main.h"

int admin_chan(aNick *nick, aChan *chan, int parc, char **parv)
{
	char *c = NULL, *raison;
	const char *arg = parv[1];
	int i = 0, m = 0;

	if(parc > 1) chan = getchaninfo((c = parv[2]));

	if(!strcasecmp(arg, "suspend"))
	{
		if(parc < 2)
			return csntc(nick, "Syntaxe: %s SUSPEND <salon> [arg]", RealCmd("chan"));

		if(!chan) return csntc(nick, NOSUCHAN, c);

		if(!IsSuspend(chan) && ((parc == 3 && *parv[3] == '%') || parc < 3))
			return csntc(nick, "Veuillez prciser une raison pour suspendre ce salon.");

		if(!IsSuspend(chan))
		{
			time_t timeout = 0;
			if(*parv[3] == '%' && !(timeout = convert_duration(++parv[3])))
				return csntc(nick, "Dure incorrecte, le format est %%<XjXhXm> X tant le nombre respectif d'units.");

			do_chan_suspend(chan, parv2msg(parc, parv, timeout ? 4 : 3, 250), nick->user->nick, timeout, time(NULL));
			show_suspend(nick, chan);
			cspart(chan, chan->suspend->raison);
			putlog(LOG_CHANS, "SUSPEND ON %s par %s@%s (%s)\n", c, nick->nick, nick->user->nick, chan->suspend->raison);
		}
		else
		{
			if(getoption("-nolog", parv, parc, 2, -1))
			{
				free(chan->suspend);
				chan->suspend = NULL;
			}
			else chan->suspend->expire = -1;

			csntc(nick, "Le salon \2%s\2 n'est plus suspendu.", c);
			csjoin(chan);
			putlog(LOG_CHANS, "SUSPEND OFF %s par %s@%s\n", c, nick->nick, nick->user->nick);
		}
		write_achan(chan);
	}
	else if(!strcasecmp(arg, "del"))
	{
		char tmp[400];
		if(parc < 2)
			return csntc(nick, "Syntaxe: %s DEL <salon> <raison>", RealCmd("chan"));

		if(!chan) return csntc(nick, NOSUCHAN, c);

		raison = (parc > 2) ? parv2msg(parc, parv, 3, 300) : "Aucune raison";
		snprintf(tmp, sizeof tmp, "Salon %s dsenregistr par %s@%s (%s)", c, nick->nick, nick->user->nick, raison);

		csntc(nick, "Le salon %s a bien t supprim.", c);
		del_chan(chan, tmp);
		write_chanlist();
	}
	else if(!strcasecmp(arg, "reg"))
	{
		anUser *u;
		if(parc < 3)
			return csntc(nick, "Syntaxe: %s REG <salon> <Username|%%nick> [description]", RealCmd("chan"));

		if(chan) return csntc(nick, "%s est dj enregistr.", c);

		if(!chan_check(c, nick)) return 0;

		if(!(u = ParseNickOrUser(nick, parv[3]))) return 0;

		raison = (parc >= 4) ? parv2msg(parc, parv, 4, 300) : "Adminreg";

		chan = add_chan(c, "", raison, "+nt", "", "", "", "");
		add_access(u, c, OWNERLEVEL, (A_OP | A_PROTECT), time(NULL));
		csjoin(chan);
		append_file(CHANS_FILE,c);
		write_achan(chan);
		write_anuser(u);
		csntc(nick, "%s a t enregistr pour %s.", c, u->nick);
	}
	else if(!strcasecmp(arg, "list"))
	{
		int description = getoption("-description", parv, parc, 2, 0);
		int modes = getoption("-modes", parv, parc, 2, 0);
		int defmodes = getoption("-defmodes", parv, parc, 2, 0);
		int url = getoption("-url", parv, parc, 2, 0), count = getoption("-count", parv, parc, 2, 1);
		int topic = getoption("-topic", parv, parc, 2, 0);

		if(parc < 2 || *parv[2] == '-') m = 1;

		for(chan=chanhead;chan;chan=chan->next)
			if((m || !match(parv[2], chan->chan))
			&& (!description || !match(parv[description], chan->description))
			&& (!modes || !match(parv[modes], GetCModes(chan)))
			&& (!defmodes || !match(parv[defmodes], chan->defmodes))
			&& (!url || !match(parv[url], chan->url))
			&& (!topic || !match(parv[topic], chan->topic))
			&& (++i <= count || !count))
			{
				if(i == 1) csntc(nick, "\2Liste des salons enregistrs :");
				if(!IsSuspend(chan)) csntc(nick, "  %s (%s/%s) [%d]", chan->chan, chan->defmodes, GetCModes(chan), chan->users);
				else csntc(nick, "  %s (%s) (\2Suspendu\2)", chan->chan, chan->defmodes);
			}

		csntc(nick, "Il y a un total de\2 %d\2 salon%s enregistr%s correspondants.", i, PLUR(i), PLUR(i));
	}
	else if(!strcasecmp(arg, "suspendlist"))
	{
		int from = getoption("-from", parv, parc, 2, 0);
		if(parc < 2) m = 1;

		for(chan=chanhead;chan;chan=chan->next)
			if(IsSuspend(chan)
				&& (m || !match(parv[2], chan->chan))
				&& (!from || !strcasecmp(parv[from], chan->suspend->from)))
			{
				if(!i++) csntc(nick, "\2Liste des salons suspendus :");
				csntc(nick, "  %s Par %s Expire %s Raison: %s",
						chan->chan, chan->suspend->from,
						chan->suspend->expire ? get_time(chan->suspend->debut + chan->suspend->expire) : "Jamais",
						chan->suspend->raison);
			}
		csntc(nick, "Il y a un total de\2 %d\2 salons suspendus correspondant  vos critres.", i);
	}
	else if(!strcasecmp(arg, "join"))
	{
		if(parc < 2) return csntc(nick, "Syntaxe: %s JOIN <salon> [-force]", RealCmd("chan"));

		if(!chan) return csntc(nick, NOSUCHAN, c);

		if(IsSuspend(chan))
			return csntc(nick, "%s est actuellement suspendu pour %s, utilisez '%s SUSPEND %s' pour lever le suspend.",
				c, chan->suspend->raison, RealCmd("chan"), c);

		if(CJoined(chan) && !getoption("-force", parv, parc, 2, -1))
			return csntc(nick, "Je suis dj sur %s (En cas de desynch, utilisez l'argument '-force')", c);

		csjoin(chan); /* nicklist restaure dans le csjoin */
	}
	else if(!strcasecmp(arg, "part"))
	{
		if(parc < 2) return csntc(nick, "Syntaxe: %s PART <salon> [raison]", RealCmd("chan"));

		if(!chan) return csntc(nick, NOSUCHAN, c);

		cspart(chan, (parc > 2) ? parv2msg(parc, parv, 3, 300) : "Aucune Raison");
	}
	else if(!strcasecmp(arg, "newowner"))
	{
		anAccess *access, *new = NULL;
		anUser *user, *newu = NULL;

		if(parc <3) return csntc(nick, "Syntaxe: %s NEWOWNER  <chan> <nouvel owner>", RealCmd("chan"));

		if(!chan) return csntc(nick, NOSUCHAN, c);

		for(user = userhead;user;user = user->next)
		{
			if(!strcasecmp(user->nick, parv[3]))
			{
				new = GetAccessIbyUserI(user,c);
				newu = user;
				if(new && new->level == OWNERLEVEL)
					return csntc(nick, "%s est dj l'owner de %s", parv[3], c);
			}
			else if((access=GetAccessIbyUserI(user,c)) && access->level == OWNERLEVEL)
			{
				csntc(nick, "Le salon %s possde dj un owner: %s", c, user->nick);
				csntc(nick, "Pour que %s ne soit plus owner et que %s le devienne, utilisez /%s %s CHOWNER %s %s %s",
					user->nick, parv[3], cs.nick, RealCmd("chan"), c, user->nick, parv[3]);
				return 0;
			}
		}

		if(!newu) return csntc(nick, NOSUCHUSER, parv[3]);

		if(!new) add_access(newu, c, OWNERLEVEL, (A_OP | A_PROTECT), time(NULL));
		else new->level = OWNERLEVEL;

		write_anuser(newu);
		csntc(nick, "%s est maintenant l'owner de %s", parv[3], c);
	}
	else if(!strcasecmp(arg, "chowner"))
	{
		anAccess *old = NULL, *new = NULL;
		anUser *user, *newu = NULL, *lastu = NULL;
		int newlevel = 0;

		if(parc <4)
			return csntc(nick, "Syntaxe: %s CHOWNER  <chan> <owner actuel> <nouvel owner> [niveau de l'ancien owner]", RealCmd("chan"));

		if(parc > 4 && is_num(parv[5])) newlevel = atoi(parv[5]);
		if(newlevel < 0 || newlevel >= OWNERLEVEL)
			return csntc(nick, "Le nouveau niveau de l'ancien owner doit tre compris entre 1 et %d (ou 0 pour supprimer l'access)", OWNERLEVEL - 1);

		if(!strcasecmp(parv[3], parv[4]))
			return csntc(nick, "Veuillez prciser un VRAI nouvel owner.");

		if(!chan) return csntc(nick, NOSUCHAN, c);

		for(user = userhead;user;user = user->next)
		{
			if(!strcasecmp(user->nick, parv[3]))
			{
				old = GetAccessIbyUserI(user,c);
				if(!old || old->level != OWNERLEVEL)
					return csntc(nick, "%s n'est pas l'owner de %s", parv[3], c);
				lastu = user;
			}
			if(!strcasecmp(user->nick, parv[4]))
			{
				new = GetAccessIbyUserI(user,c);
				if(new && new->level == OWNERLEVEL)
					return csntc(nick, "%s est dj l'owner de %s", parv[4], c);
				newu = user;
			}
			if(newu && lastu) break;
		}

		if(!newu || !lastu)
		{
			if(!newu && !lastu)
				csntc(nick, "\2%s\2 et \2%s\2 ne sont pas des usernames existants.", parv[3], parv[4]);
			else csntc(nick, NOSUCHUSER, !newu ? parv[4] : parv[3]);
			return 0;
		}

		if(!new) add_access(newu, c, OWNERLEVEL, (A_OP | A_PROTECT), time(NULL));
		else new->level = OWNERLEVEL;

		if(!newlevel) del_access(lastu, chan);
		else old->level = newlevel;

		write_anuser(lastu);
		write_anuser(newu);

		if(newlevel) csntc(nick, "%s est maintenant l'owner de %s, et %s a un access de %d", parv[4], c, parv[3], newlevel);
		else csntc(nick, "%s est maintenant l'owner de %s, et %s n'y a plus access", parv[4], c, parv[3]);
	}
#ifdef SQLLOG
	else if(!strcasecmp(arg, "historique"))
	{
		time_t date = 0;
		MYSQL_RES *result;
		MYSQL_ROW row;
		int cmd, user;
		char tmp[150] = "SELECT * FROM chancmdslog WHERE chan='%s' AND TS>=%T";

		if(parc < 2) return csntc(nick, "Syntaxe: %s HISTORIQUE <salon> [%%XjXh] [-cmd <cmd>] [-user <user>]", RealCmd("chan"));
		if(!chan) return csntc(nick, NOSUCHAN, c);
		if(parc >= 3 && *parv[3] == '%' && !(date = convert_duration(++parv[3])))
			return csntc(nick, "Dure incorrecte, le format est %%<XjXhXm> X tant le nombre respectif d'units.");

		if((cmd = getoption("-cmd", parv, parc, 2, 0))) strcat(tmp, " AND cmd='%s'");
		if((user = getoption("-user", parv, parc, 2, 0))) strcat(tmp, " AND user='%s'");

		if(sql_query(tmp, chan->chan, date ? time(NULL) - date : 0,
			cmd ? parv[cmd] : (user ? parv[user] : ""), user ? parv[user] : "") < 0) return csntc(nick, "La requte SQL a chou.");

		if(!(result = mysql_store_result(bot.sql_id)) || !(row = mysql_fetch_row(result)))
			return csntc(nick, "Aucune entre trouve.");

		if(date) csntc(nick, "Commandes effectues sur \2%s\2 (Par %s) depuis %s.", chan->chan, user ? parv[user] : "tout le monde", duration(date));
		else csntc(nick, "Commandes effectues sur \2%s\2 (Par %s).", chan->chan, user ? parv[user] : "tout le monde");

		for(;row;row = mysql_fetch_row(result)) csntc(nick, "%s: \2%s\2 [%s] par \2%s\2", get_time(strtoul(row[2], NULL, 10)), row[3], NONE(row[4]), row[0]);
		mysql_free_result(result);
	}
#endif
	else return csntc(nick, "%s n'est pas une option valide.", arg);
	return 1;
}

int whoison(aNick *nick, aChan *chan, int parc, char **parv)
{
	int first = 1, size = 0;
	char buf[451] = {0}, tmp[NICKLEN + 16];
	aLink *lp = chan->members;

	if(!lp) return csntc(nick, "Aucun User sur %s.", chan->chan);
	for(;lp;lp=lp->next)
	{
		int tmps = size, p = snprintf(tmp, (NICKLEN + 15), " %s%s", GetChanPrefix(lp->value.j->nick, lp->value.j, 1), lp->value.j->nick->nick);
		size += p > 0 ? p : NICKLEN + 15;;
		if(size >= 450)
		{
			if(first) csntc(nick, "Un total de %d Users sur %s:%s...", chan->users, chan->chan, buf);
			else csntc(nick, "              :%s...", buf);
			first = 0;
			strcpy(buf, tmp);
			size = strlen(tmp);
		}
		else strcpy(&buf[tmps], tmp);
	}
	if(first && buf[0] != '\0') csntc(nick, "Un total de %d Users sur %s:\2%s", chan->users, chan->chan, buf);
	else if(buf[0] != '\0') csntc(nick, "              :%s", buf);

	return 1;
}

void do_chan_suspend(aChan *chan, const char *raison, const char *nick, time_t expire, time_t debut)
{
	if(!chan->suspend) chan->suspend = calloc(1, sizeof(struct suspendinfo));
	Strncpy(chan->suspend->raison, raison, 250);
	Strncpy(chan->suspend->from, nick, NICKLEN);
	chan->suspend->expire = expire;
	chan->suspend->debut = debut;
	return;
}

void show_suspend(aNick *nick, aChan *chan)
{
	if(!chan->suspend) return; /* on aurait jamais du l'envoyer l*/
	if(IsSuspend(chan))
	{
		char buf[200];
		Strncpy(buf, get_time(chan->suspend->debut), sizeof buf -1);
		csntc(nick, "Le salon %s \2est\2 suspendu par %s (le %s). Expire %s",
			chan->chan, chan->suspend->from, buf,
			chan->suspend->expire ? get_time((chan->suspend->debut + chan->suspend->expire)) : "Jamais");
	}
	else csntc(nick, "Le salon tait suspend par %s", chan->suspend->from);
	csntc(nick, "Raison: %s", chan->suspend->raison);
	return;
}
