/* src/admin_cmds.c - Diverses commandes pour admins
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "main.h"
#include "web2cs.h"

CVSINFO("$Id: admin_cmds.c,v 1.35 2003/08/04 21:41:03 romexzf Exp $");

extern const int lastresetparse;
extern const int lastresetlogs;
extern const int kill_interval;
extern int want_to_quit;
extern char *quit_msg;
extern char *pasdeperm;
extern char *mailprog;
extern char *network;
extern struct killinfo *killhead;
extern struct ignore *ignorehead;
extern struct Link *mainhub;
extern int ignoretime;
extern int maxlastseen;
extern char HIDDEN_HOST[];
#ifdef TESTBOT
extern int w2c_port;
extern w2c_hallow *w2c_hallowhead;
#endif
#ifdef USE_WELCOMESERV
extern char user_motd[];
extern char admin_motd[];
#endif

extern struct badchans *badchead;
extern struct badnicks *badnhead;

extern struct cs_commands cs_cmds[];
extern int list_badchans(aNick *, aChan *, int, char **);
extern int list_badnicks(aNick *, aChan *, int, char **);

/* invite des admins sur le salon d'infos */
int inviteme(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
    putserv("%s " TOKEN_INVITE " %s :%s\r\n", cs.num, nick->nick, bot.pchan);
    return 1;
}

void clearallmem(void)
{
	aNick *n, *nt;
	aJoin *j, *jt;
	aChan *c, *ct;
	aBan *b, *bt;
	anUser *u, *ut;
	anAccess *a, *at;
#ifdef USE_MEMOSERV
	aMemo *m, *mt;
#endif
	struct badchans *bc, *bct;
	struct badnicks *bn, *bnt;
	struct ignore *ii, *it;
	aKill *k, *kt;
	struct Link *l, *lt;

	free(quit_msg);
	free(mailprog);
	free(pasdeperm);
	free(network);

	for(bc=badchead;bc;bc=bct)
	{
		bct=bc->next;
		free(bc);
	}
	for(bn=badnhead;bn;bn=bnt)
	{
		bnt=bn->next;
		free(bn);
	}
	for(c=chanhead;c;c=ct)
	{
		ct=c->next;
		for(b=c->banhead;b;b=bt)
		{
			bt=b->next;
			free(b);
		}
		if(c->suspend) free(c->suspend);
		if(c->motd) free(c->motd);
		free(c);
	}
	for(u=userhead;u;u=ut)
	{
		ut=u->next;
		for(a=u->accesshead;a;a=at)
		{
			at=a->next;
			if(a->info) free(a->info);
			free(a);
		}
#ifdef USE_MEMOSERV
		for(m=u->memohead;m;m=mt)
		{
			mt=m->next;
			free(m);
		}
#endif
		if(u->suspend) free(u->suspend);
		if(u->lastlogin) free(u->lastlogin);
		free(u);
	}
	for (n=nickhead;n;n=nt)
	{
		nt=n->next;
		for(j=n->joinhead;j;j=jt)
		{
			jt=j->next;
			free(j);
		}
		free(n);
	}
	for(k=killhead;k;k=kt)
	{
		kt=k->next;
		free(k);
	}
	for(ii=ignorehead;ii;ii=it)
	{
		it=ii->next;
		free(ii);
	}
	for(l=mainhub;l;l=lt)
	{
		lt=l->next;
		free(l);
	}
	return;
}

int die(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char *r = parc ? parv2msg(parc, parv, 1, 450) : quit_msg;

	want_to_quit = 1;
	write_cmds();

	putserv("%s " TOKEN_QUIT " :%s (par \2%s\2)\n", cs.num, r, nick->nick);

#ifdef USE_WELCOMESERV
	putserv("%s " TOKEN_QUIT " :%s (par \2%s\2)\n", ws.num, r, nick->nick);
#endif

	putserv("%s " TOKEN_SQUIT " %s 0 :%s (par \2%s\2)\n", bot.servnum, bot.server, r, nick->user->nick);
	remove(SCODERZ_PID);
	clearallmem();
	return 1;
}

/*
 * restart_bot
 *
 * parv[1->parc-1] = reason
 */
int restart_bot(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	FILE *fuptime;
	char *r = parc ? parv2msg(parc, parv, 1, 450) : quit_msg;

	if((fuptime = fopen("uptime.tmp", "w"))) /* permet de maintenir l'uptime (/me tricheur)*/
	{
		fprintf(fuptime, "%lu %d %d", bot.uptime, lastresetparse, lastresetlogs);
		fclose(fuptime);
	}

	want_to_quit = 1;
	write_cmds();

	putserv("%s " TOKEN_QUIT " :%s [\2RESTART\2] (par \2%s\2)\n", cs.num, r, nick->nick);
#ifdef USE_WELCOMESERV
	putserv("%s " TOKEN_QUIT " :%s [\2RESTART\2] (par \2%s\2)\n", ws.num, r, nick->nick);
#endif

	putserv("%s " TOKEN_SQUIT " %s 0 :%s [\2RESTART\2] (par \2%s\2)\n", bot.servnum, bot.server, r, nick->user->nick);

	clearallmem();
	remove(SCODERZ_PID);
#ifdef TESTBOT
	w2c_closesocks();
#endif
	system("./services -h");
	return 1;
}

int cs_wall(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	aNick *n;
	char *buf = parv2msg(parc, parv, 1, 400);

	for(n=nickhead;n;n=n->next)
		if(IsAnAdmin(n->user)) csntc(n, "CSWall de \2%s\2: %s", nick->nick, buf);
	return 1;
}

int chcomname(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char newcmd[15] = {0}, *lastcmd = parv[1];
	int i, icmd;

	if(strlen(parv[2]) > 14)
		return csntc(nick, "Un nom de commande ne doit pas dpasser 14 caractres");
	strcpy(newcmd, parv[2]);

	if((icmd = GetCurCmdIndexbyName(lastcmd)) < 0)
		return csntc(nick, "La commande\2 %s\2 existe pas.", lastcmd);

	if(cs_cmds[icmd].cmd[0] == '\1')
		return csntc(nick, "Vous ne pouvez pas changer le nom des commandes en ctcp");

	if((i = GetCurCmdIndexbyName(newcmd)) >= 0)
		return csntc(nick, "La commande %s existe dj.", newcmd);

	strcpy(cs_cmds[icmd].cmd, newcmd);
	csntc(nick, "La commande\2 %s\2 s'appelle maintenant\2 %s\2.", lastcmd, cs_cmds[icmd].cmd);
	write_cmds();
	return 1;
}

int chlevel(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	const char *cmd = parv[1];
	int level, i, adm = getoption("-admin", parv, parc, 2, -1);

	if((i = GetCurCmdIndexbyName(cmd)) < 0)
		return csntc(nick, "La commande\2 %s\2 existe pas.", cmd);

	if(!is_num(parv[2]) || atoi(parv[2]) > OWNERLEVEL || atoi(parv[2]) < 0)
		return csntc(nick, "Un niveau doit tre un nombre entre 0 et %d", OWNERLEVEL);

	if((level = atoi(parv[2])) == cs_cmds[i].level && !adm)
		return csntc(nick, "%d est actuellement le niveau de %s.", level, cmd);

	if(!ChanCmd(i))
	{
		if(level > 4) return csntc(nick, "Le niveau d'une commande user ne doit pas dpasser 4.");
		else if(level > nick->user->level)
			return csntc(nick, "\2Vous ne pouvez pas donner un niveau  %s plus grand que le votre.\2", cmd);
		else
		{ /* dtermination du niveau admin ou pas car level 0-4 */
			if(level > 1) cs_cmds[i].flag |= CMD_ADMIN;
			else cs_cmds[i].flag &= ~CMD_ADMIN;
		}
	}
	if(ChanCmd(i) && adm) /* argument de switch admin pour les cmds chans (0-500 <= donc ncessaire)*/
	{
		if(AdmCmd(i)) cs_cmds[i].flag &= ~CMD_ADMIN;
		else cs_cmds[i].flag |= CMD_ADMIN;
	}

	cs_cmds[i].level = level;

	csntc(nick, "Le niveau de la commande %s est maintenant\2 %d\2 (%s).",
				cs_cmds[i].cmd, cs_cmds[i].level, AdmCmd(i) ? "Admin" : "User");
	write_cmds();
	return 1;
}

int disable_cmd(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	const char *cmd = parv[1];
	int disable = 0, i = 0;

	if(!strcasecmp(parv[2], "ON")) disable = 1;
	else if(!strcasecmp(parv[2], "OFF")) disable = 0;
	else return csntc(nick, "Vous devez prciser la commande est desactive grace  ON ou OFF.");

	for(;cs_cmds[i].cmd[0] != '\0';i++)
		if(!strcasecmp(cmd, cs_cmds[i].cmd))
		{
			if(!strcasecmp(cs_cmds[i].realcmd, "DISABLE")
			|| !strcasecmp(cs_cmds[i].realcmd, "CHCOMNAME")
			|| !strcasecmp(cs_cmds[i].realcmd, "CHLEVEL")
			|| !strcasecmp(cs_cmds[i].realcmd, "LOGIN")) /* a nuirait  tout.. */
				return csntc(nick, "Veuillez ne pas desactiver \2%s\2 !", cmd);

			if(disable) cs_cmds[i].flag |= CMD_DISABLE;
			else cs_cmds[i].flag &= ~CMD_DISABLE;

			csntc(nick, "La commande %s est maintenant \2%sactive\2", cs_cmds[i].cmd, disable ? "des" : "");
			write_cmds();
			return 1;
		}

	return csntc(nick, "La commande\2 %s\2 existe pas.", cmd);
}

#ifdef USE_WELCOMESERV
int set_motds(aNick *nick, aChan *c, int parc, char **parv)
{
	if(parc < 2) return syntax_cmd(nick, GetCmdIndexbyName("adminmotd"));

	if(!strcasecmp(parv[1], "USER"))
	{
		if(!strcasecmp("none", parv[2])) user_motd[0] = 0;
		else parv2msgn(parc, parv, 2, user_motd, 390);
	}
	else if(!strcasecmp(parv[1], "ADMIN"))
	{
		if(!strcasecmp("none", parv[2])) admin_motd[0] = 0;
		else parv2msgn(parc, parv, 2, admin_motd, 390);
	}
	else if(!strcasecmp(parv[1], "LIST"))
	{
		csntc(nick, "Motd User : \2%s\2", user_motd);
		csntc(nick, "Motd Admin: \2%s\2", admin_motd);
		return 1;
	}
	else return csntc(nick, "Il n'y a pas de motd pour %s", parv[1]);

	if(strcasecmp("none", parv[2])) csntc(nick, "Le motd pour les %ss est maintenant:\2 %s",
							parv[1], !strcasecmp(parv[1], "USER") ? user_motd : admin_motd);
	else csntc(nick, "Le motd pour les %ss a bien t supprim", parv[1]);
	write_welcome();
	return 1;
}
#endif

/* cmds (c) keeo (le 3 en 1 par progs)*/
int globals_cmds(aNick *nick, aChan *chan, int parc, char **parv)
{
	const char *cmd = parv[1];
	char *msg, msg2[400] = {0};

	if(strcasecmp(cmd, "MEMO")) parv2msgn(parc, parv, 2, msg2, 400);

	msg = (strcasecmp(cmd, "MEMO")) ? msg2 : NULL;
	if(msg && *msg == ':') msg++;

	if(!strcasecmp(cmd, "WALL"))
	{
		aNick *n = nickhead;
		for(;n;n=n->next) csntc(n, "[\2Notice Globale\2] %s - \002%s\2", msg, nick->nick);
	}
	else if(!strcasecmp(cmd, "MSG"))
	{
		for(chan = chanhead;chan;chan=chan->next)
		  if(chan->users > 1) putserv("%s " TOKEN_PRIVMSG " %s :\2%s >\2 %s\n", cs.num, chan->chan, nick->nick, msg);
	}
	else if(!strcasecmp(cmd, "CNOTICE"))
	{
		for(chan = chanhead;chan;chan=chan->next)
		  if(chan->users > 1) putserv("%s " TOKEN_NOTICE " %s :%s - \002%s\2\n", cs.num, chan->chan, msg, nick->nick);
	}
#ifdef USE_MEMOSERV
	else if(!strcasecmp(cmd, "MEMO"))
	{
		char memo[MEMOLEN + 1];
		anUser *user;
		aNick *who;
		int min_level = 0, max_level = 0, mask = 0, i = 2, owner;

		for(;i;i++)
		{
			if(*parv[i] == '-')	i++;
			else
			{
				parv2msgn(parc, parv, i, msg2, MEMOLEN);
				break;
			}
		}
		strncpy(memo, msg2, MEMOLEN);
		min_level = getoption("-min",parv,parc,2,1);
		max_level = getoption("-max",parv,parc,2,1);
		owner = getoption("-owner", parv, parc, 2, -1);

		if(max_level > 4 || min_level > 4)
			return csntc(nick, "Les niveaus maximum ou minimum  spcifier doivent tre compris entre 1 et 4 (c'est les niveau d'users)");

		mask = getoption("-user",parv,parc,2,0);

		for(user = userhead;user;user = user->next)
			if(user->nick != nick->user->nick && (!min_level || user->level >= min_level)
			   && (!max_level || user->level <= max_level) && (!mask || !match(parv[mask], user->nick))
			   && (!owner || IsAnOwner(user)))
			{
				who = user->n;
				if(who && !(who->flag & N_AWAY))
					csntc(who, "\2MEMO DE %s :\2 %s", nick->nick, memo);
				add_memo(user, nick->user->nick, time(NULL), memo, (who && !(who->flag & N_AWAY)) ? 1 : 0);
			}

		csntc(nick, "Un memo a bien t envoy  tout le monde respectant les critres donns (si spcifis) avec le texte suivant :");
		csntc(nick, "  \002%s\2", memo);
		write_users();
	}
#endif
	else
	{
		csntc(nick, "La sous commande %s n'existe pas", cmd);
		syntax_cmd(nick, GetCmdIndexbyName("GLOBAL"));
	}
	return 1;
}


/*
 * del_badchan
 *
 * parv[1] = chaine
 */
int del_badchan(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	const char *chaine = parv[1];
	struct badchans *badchan;

	if(!(badchan = getbadchan(chaine))) return csntc(nick, "La chaine %s n'existe pas", chaine);

	del_abadchan(badchan->chaine);
	csntc(nick, "La chaine\2 %s\2 a bien t supprime", chaine);
	write_badchans();
	return 1;
}

/*
 * add_badchan
 *
 * parv[1] = chaine
 */
int add_badnick(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char chaine[NICKLEN + 1];
	struct badnicks *badnick;
	int badn = getoption("-badnick", parv, parc, 1, -1), i = 0;
	anUser *u = userhead;

	if((badnick = getbadnick(parv[1])))
		return csntc(nick, "Une chaine a dj t enregistre contenant celle ci:\2 %s\2.", badnick->chaine);

	strncpy(chaine, parv[1], NICKLEN);
	csntc(nick, "\002%s\2 est maintenant un %s illgal", chaine, badn ? "pseudo" : "username");

	add_abadnick(chaine, !!badn);
	if(!badn) csntc(nick, "L'username %s n'est dsormais plus enregistrable", chaine);
	else csntc(nick, "Tous les users qui voudront prendre ce pseudo seront deconnects au bout de %d secondes.", kill_interval);

	for(;u;u = u->next) if(!match(chaine, u->nick)) i++;
	if(i) csntc(nick, "Actuellement %d UserNames sont couverts par '%s'", i, chaine);

	append_file(BADNICKS_FILE, chaine);
	return 1;
}

/*
 * del_badnick
 *
 * parv[1] = chaine
 */
int del_badnick(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char chaine[NICKLEN + 1];
	struct badnicks *badnick;

	strncpy(chaine, parv[1], NICKLEN);

	if(!(badnick = getbadnick(chaine))) return csntc(nick, "La chaine %s n'existe pas", chaine);

	del_abadnick(badnick->chaine);
	csntc(nick, "La chaine\2 %s\2 a bien t supprime", chaine);
	write_badnicks();
	return 1;
}

int badmanage(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	const char *type = parv[1], *cmd = parv[2];

	if(strcasecmp(type,"nick") && strcasecmp(type, "chan"))
	{
		csntc(nick ,"Invalide type: veuillez choisir 'nick' ou 'chan'");
		return syntax_cmd(nick, GetCmdIndexbyName("abad"));
	}
	if(strcasecmp(cmd,"del") && strcasecmp(cmd,"add") && strcasecmp(cmd, "list") && strcasecmp(cmd, "noreg"))
	{
		csntc(nick ,"Invalide commande: veuillez choisir 'del', 'add', 'noreg' ou 'list'");
		 return syntax_cmd(nick, GetCmdIndexbyName("abad"));
	}
	if(!strcasecmp(type,"chan"))
	{
		if(!strcasecmp(cmd,"add"))
		{
			char chaine[100] = {0}, raison[150] = {0}, tmp[255];
			struct badchans *badchan;

			if(parc<4) return csntc(nick, "Syntaxe: %s CHAN ADD <mask/chan> <raison>", RealCmd("abad"));

			if((badchan = getbadchan(parv[3])))
				return csntc(nick, "%s est dj couvert par\2 %s\2 (\002%s\2)\2.", parv[3], badchan->chaine, badchan->raison);

			strncpy(chaine, parv[3], 99);
			parv2msgn(parc, parv, 4, raison, 149);/*pas de boucles inutiles avt les check*/

			badchan = add_abadchan(chaine, raison);
			csntc(nick, "La chaine\2%s\2 a bien t ajoute avec la raison:\2 %s", chaine, raison);
			csntc(nick, "Aucun salon contenant cette chaine ne pourra tre enregistr par les Users.");

			snprintf(tmp, 255, "%s :%s", badchan->chaine, badchan->raison);
			append_file(BADCHANS_FILE, tmp);
		}
		else if(!strcasecmp(cmd,"del"))
		{
			if(parc < 3) return csntc(nick, "Syntaxe: %s CHAN DEL <mask/chan> <raison>", RealCmd("abad"));

			parv[1] = parv[3];
			del_badchan(nick,chaninfo,parc,parv);
		}
		else if(!strcasecmp(cmd,"list")) list_badchans(nick, chaninfo, parc, parv);
		return 1;
	}
	if(!strcasecmp(type,"nick"))
	{
		if(!strcasecmp(cmd,"add"))
		{
			if(parc<3) return csntc(nick, "Syntaxe: %s USER ADD <mask/nick>", RealCmd("abad"));

			parv[1] = parv[3];
			add_badnick(nick,chaninfo,parc,parv);
		}
		else if(!strcasecmp(cmd,"del"))
		{
			if(parc < 3) return csntc(nick, "Syntaxe: %s USER ADD DEL <mask/user>", RealCmd("abad"));

			parv[1] = parv[3];
			del_badnick(nick,chaninfo,parc,parv);
		}
		else if(!strcasecmp(cmd,"noreg"))
		{
			struct badnicks *badnick;
			if(parc < 3) return csntc(nick, "Syntaxe: ABAD USER NOREG <mask/nick>", RealCmd("abad"));

			if(!(badnick = getbadnick(parv[3])))
				return csntc(nick, "%s n'est pas un badnick enregistr.", parv[3]);

			ToggleFlag(&badnick->flag, CF_NOREG);
			csntc(nick, "%s est maintenant un %s interdit", badnick->chaine,
				(badnick->flag & CF_NOREG) ? "username" : "pseudo");
		}
		else if(!strcasecmp(cmd,"list")) list_badnicks(nick, chaninfo, parc, parv);
		return 1;
	}
	return 0;
}

int rehash_conf(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	if(getoption("-help", parv, parc, 1, -1))
	{
		int i = 0;
		FILE *f;
		char tmp[15], buf[250], dir[50];

		for (; cs_cmds[i].cmd[0] != '\0'; i++)
		{
			while(cs_cmds[i].cmd[0] == '\001') i++;
			strncpy(tmp, cs_cmds[i].cmd, 15);
			strupr(tmp);
			snprintf(dir, 50, "syntax/%s", tmp);
			if((f=fopen(dir, "r")))
			{
				fgets(buf, 250, f);
				strcpy(cs_cmds[i].syntax, buf);
				fclose(f);
			}
		}
		return csntc(nick, "Syntaxe recharge depuis le HDD.");
	}
	if (load_config(FICHIER_CONF) == -1)
	{
		csntc(nick, "Impossible d'ouvrir la configuration");
		return csntc(nick, "Il est possible qu'une erreur ait t faite dans la configuration, vrifiez.");
	}

	return csntc(nick, "Configuration actualise avec succs");
}

int showconfig(aNick *nick, aChan *c, int parc, char **parv)
{
#ifdef TESTBOT
	w2c_hallow *halw;
#endif
	csntc(nick, "Etat des variables de configuration");
	csntc(nick, "Nom du rseau : %s", network);
	csntc(nick, "Nom du serveur des Services : %s (%s) [Num:%d]", bot.server, bot.name, bot.num);
	csntc(nick, "IP du link : %s Port %d:", bot.ip, bot.port);
	csntc(nick, "Salon de controle : %s Salon d'aide : %s", bot.pchan, bot.chan);
	csntc(nick, "Message de quit : %s", quit_msg);
	csntc(nick, "Message PASDEPERM : %s", pasdeperm);
	csntc(nick, "Temps avant un kill : %d", kill_interval);
	csntc(nick, "Change nick : %s", GetConf(CF_CHANGENICK) ? "OUI" : "NON");
	csntc(nick, "Max LastSeen : %s", duration(maxlastseen));
	csntc(nick, "Admin exempt de flood : %s", GetConf(CF_ADMINEXEMPT) ? "OUI" : "NON");
	csntc(nick, "Kill pour flood : %s", GetConf(CF_KILLFORFLOOD) ? "OUI" : "NON");
	csntc(nick, "Temps d'ignore : %d secondes", ignoretime);
	csntc(nick, "Programme de mail : %s", mailprog);
	csntc(nick, "Host hidding activ : %s Host cach : %s", GetConf(CF_HOSTHIDING) ? "OUI" : "NON", HIDDEN_HOST);
	csntc(nick, "XMODE : %s HIDECODER : %s", GetConf(CF_XMODE) ? "OUI" : "NON", GetConf(CF_HIDE_ADML4) ? "OUI" : "NON");
#ifdef TESTBOT
	csntc(nick, "Web2Cs/Port : %d", w2c_port);
	for(halw=w2c_hallowhead;halw; halw=halw->next)
		csntc(nick, "Web2Cs/HostAllow : %s", halw->host);
#endif
	return 1;
}



