/* src/web2cs-engine.c - Moteur du Web2CS
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "main.h"
#ifdef TESTBOT
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdarg.h>
#include <fcntl.h>
#include <sys/time.h>
#include <errno.h>
#include "web2cs.h"

CVSINFO("$Id: web2cs-engine.c,v 1.19 2003/06/30 20:19:59 romexzf Exp $");

extern struct w2c_commands w2c_cmds[];
extern const int w2c_port;
int bind_sock;
int CONtotal = 0;
int WEBtrafficUP = 0;
int WEBtrafficDL = 0;
fd_set socks;        /* Socket file descriptors we want to wake up for, using select() */
fd_set master;
int highsock;        /* Highest #'d file descriptor, needed for select() */
struct w2c_clientinfo *w2c_clihead = NULL;
struct w2c_host_allow *w2c_hallowhead = NULL;
typedef struct w2c_clieninfo w2c_cli;

void w2c_setnonblocking(int sock)
{
	int opts = fcntl(sock,F_GETFL);

	if (opts < 0)
	{
		putlog(LOG_ERREURS, "Erreur lors de fcntl(F_GETFL)");
		exit(EXIT_FAILURE);
	}
	opts = (opts | O_NONBLOCK);
	if (fcntl(sock,F_SETFL,opts) < 0)
	{
		putlog(LOG_ERREURS, "Erreur lors de fcntl(F_SETFL)");
		exit(EXIT_FAILURE);
	}
	return;
}

void w2c_closesocks(void)
{
	int i = 0;

	socks = master;

	for(;i <= highsock;i++)
		if (FD_ISSET(i, &socks)) close(i);

	FD_ZERO(&socks);
	FD_ZERO(&master);
	return;
}

void w2c_initsocket(void)
{
	int reuse_addr = 1;
	struct sockaddr_in server_address; /* bind info structure */

	bind_sock = socket(AF_INET, SOCK_STREAM, 0); /* on demande un socket */
	if (bind_sock < 0)
	{
		putlog(LOG_ERREURS, "Impossible de crer le serveur de DB.");
		printf("Impossible de crer le serveur de DB, exiting\n");
		exit(EXIT_FAILURE);
	}
	fcntl(bind_sock, F_SETFL, O_NONBLOCK);

	setsockopt(bind_sock, SOL_SOCKET, SO_REUSEADDR, &reuse_addr, sizeof(reuse_addr));

	memset((char *) &server_address, 0, sizeof(server_address));
	server_address.sin_family = AF_INET;
	server_address.sin_addr.s_addr = INADDR_ANY;
	server_address.sin_port = htons(w2c_port);

	if (bind(bind_sock, (struct sockaddr *) &server_address, sizeof(server_address)) < 0 )
	{
		printf("Impossible d'couter au port %d (Dj pris?)\n", w2c_port);
		printf("Impossible de crer le serveur de DB, exiting\n");
		putlog(LOG_ERREURS, "Impossible d'couter au port de DB.");
		close(bind_sock);
		exit(EXIT_FAILURE);
	}

	listen(bind_sock,5);

	highsock = bind_sock;
	FD_ZERO(&master);
	FD_ZERO(&socks);

	FD_SET(bind_sock, &master);
	socks = master;
	printf("Serveur de base de donnes du Channel Service OK (Port %d)\n", w2c_port);
	return;
}

int w2c_checksocks(void)
{
	int readsocks;
	struct timeval timeout;

	timeout.tv_sec = 0;
	timeout.tv_usec = 100;

	socks = master;
	readsocks = select(highsock+1, &socks, NULL, NULL, &timeout);

	if (readsocks == -1 && errno != EINTR)
	{
		putlog(LOG_ERREURS, "Erreur lors de select()\n");
		exit(EXIT_FAILURE);
	}
	else if (readsocks <= 0) return 0;
	return 1;
}

struct w2c_clientinfo *w2c_addclient(int i, char *a1)
{
	struct w2c_clientinfo *new, *tmp;

	if(!(new = calloc(1, sizeof(struct w2c_clientinfo)))) return NULL;

	new->next = NULL;
	new->socket = i;
	new->user = NULL;

	strcpy(new->ip, a1);
	new->flag = 0;
	if(!w2c_clihead) w2c_clihead = new;
	else
	{
		for(tmp = w2c_clihead; tmp->next; tmp = tmp->next);
		if(!tmp->next) tmp->next = new;
	}

	return new;
}

void w2c_delclient(struct w2c_clientinfo *del)
{
	struct w2c_clientinfo *last = NULL, *next = w2c_clihead;

	while(next)
	{
		if(next == del) break;
		last = next;
		next = next->next;
	}
	if(next)
	{
		if(next == w2c_clihead) w2c_clihead = w2c_clihead->next;
		else last->next = next->next;

		close(next->socket);
		FD_CLR(next->socket, &master);
		free(del);
		del = NULL;
	}
	return;
}

struct w2c_clientinfo *w2c_findclient(int i)
{
	struct w2c_clientinfo *cli = w2c_clihead;
	for(;cli && cli->socket != i;cli=cli->next);
	return cli;
}

void w2c_parsemsg(struct w2c_clientinfo *source, char *msg)
{
	char *parv[256], buf[512] = {0}, *ptr, *cmd;
	int parc = 1, i = 0;
	aChan *chaninfo = NULL;

	strip_newline(msg);
	strncpy(buf, msg, 512);

	cmd = strtok(buf, " ");
	if(!cmd) cmd = msg;
	else
		for(; (ptr=strtok(NULL, " ")) && parc < 256 ; parc++)
		{
			if(strlen(ptr) < 256) parv[parc] = ptr;
			else
			{
				w2c_sendrpl(source, RPL_101);
				return;
			}
		}

	parc--;

	for (; w2c_cmds[i].func != NULL; i++)
		if (!strcasecmp(w2c_cmds[i].cmd,cmd))
		{
			if(!(source->flag & W2C_ALLOWED) && strcasecmp(cmd, "PASS"))
			{
				w2c_exit_client(source, "Non autoris");
				return;
			}
			if(parc < w2c_cmds[i].args)
			{
				w2c_sendrpl(source, RPL_102);
				return;
			}
			if((!source->user && w2c_cmds[i].level) || (source->user && w2c_cmds[i].level > source->user->level))
			{
				w2c_sendrpl(source, RPL_107);
				return;
			}
			if(w2c_cmds[i].chan && parc && !(chaninfo=getchaninfo(parv[1])))
			{
				w2c_sendrpl(source, RPL_106, parv[1]);
				return;
			}
			w2c_cmds[i].func(source, chaninfo, parc, parv);
			return;
		}

	w2c_sendrpl(source, RPL_103);
	return;
}


/*
 * Dbut : parv[1]
 * Fin : parv[parc]
 */

int w2c_sendrpl(struct w2c_clientinfo *dest, const char *pattern, ...)
{
	char buf[1024] = {0};
	va_list vl;
	int len;

	va_start(vl, pattern);
	vsnprintf(buf, 1024, pattern, vl);
	va_end(vl);

	len = strlen(buf);
	WEBtrafficUP += len;

	return send(dest->socket, buf, len, 0);
}

int w2c_exit_client(struct w2c_clientinfo *client, const char *msg)
{
	w2c_sendrpl(client, RPL_001, msg ? msg : "bye");
	w2c_delclient(client);
	for(client=w2c_clihead;client;client=client->next)
		if(client->socket > highsock) { highsock = client->socket; break; }
	return 0;
}

#endif
